<?php
include_once 'XML/Serializer.php';
include_once 'XML/Unserializer.php';
// Collection of classes to manage feeds and feed stories.

class Amalgamate {
    private $_dbh;
    
    public function __construct(&$dbh) {
	$this->_dbh = $dbh;
    }

    public function refresh() {
	$feeds = $this->getFeeds();
	$cnt = @count($feeds);
	
	for ($i = 0; $i < $cnt; $i++) {
	    $feeds[$i]->refresh();
	}
	return;
    }

    public function saveSource($src, $srclink) {
	try {
	    $sth = $this->_dbh->prepare("SELECT save_source(?, ?)");
	    $sth->execute(array($src, $srclink));
	    $sth->closeCursor();
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function getCategories() {
	$ret = array();
	try {
	    $sth = $this->_dbh->prepare("SELECT * FROM get_categories()");
	    $sth->execute();
	    
	    while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
		$ret[] = $row;
	    }
	    $sth->closeCursor();
	    return $ret;
	} catch (PDOException $e) {
	    throw $e;
	}
    }

    public function getSources() {
	$ret = array();
	try {
	    $sth = $this->_dbh->prepare("SELECT * FROM get_sources()");
	    $sth->execute();
	    while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
		$ret[] = $row;
	    }
	    $sth->closeCursor();
	    return $ret;
	} catch (PDOException $e) {
	    throw $e;
	}
    }

    public function getFeeds() {
	try {
	    $this->_dbh->beginTransaction();
	    $this->_dbh->query("SELECT * FROM get_feeds('feeds')");
	    $sth = $this->_dbh->prepare('FETCH ALL IN "feeds"');
	    $sth->execute();
	    
	    
	    while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
		$feed =& new AmalgamateFeed($this->_dbh);
		$feed->feedid = $row['feedid'];
		$feed->name = $row['name'];
		$feed->url = $row['url'];
		$feed->createddate = $row['createddate'];
		$feed->lastmodifieddate = $row['lastmodifieddate'];
		$feed->source = $row['source'];
		$feed->active = $row['active'];
		$feed->category = $row['category'];
		$ret[] = $feed;
	    }
	    $this->_dbh->commit();
	    return $ret;
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function getStories($limit = 50) {
	$ret = array();
	
	try {
	    $this->_dbh->beginTransaction();
	    $sth = $this->_dbh->prepare("SELECT * FROM get_stories('stories', ?)");
	    $sth->execute(array($limit));
	    $sth->closeCursor();
	    $sth = $this->_dbh->prepare('FETCH ALL IN "stories"');
	    $sth->execute();
	    
	    while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
		$s =& new AmalgamateFeedStory($this->_dbh);
		$s->load($row);
		$ret[] = $s;
	    }
	    
	    $sth->closeCursor();
	    $this->_dbh->commit();
	} catch (PDOException $e) {
	    throw $e;
	}
	return $ret;
    }
}

class AmalgamateFeed {
    private $_dbh;
    private $_sth;
    
    public $feedid;
    public $name;
    public $url;
    public $active;
    public $createddate;
    public $source;
    
    function __construct(&$dbh, $feedid = '') {
	$this->_dbh = $dbh;
	$this->feedid = $feedid;
	
	if ($this->feedid) {
	    $this->load();
	}
    }
    
    public function delete() {
	if (!$this->feedid) {
	    return;
	}
	
	try {
	    $this->_dbh->beginTransaction();
	    $sth = $this->_dbh->prepare("SELECT delete_feed(?)");
	    $sth->execute(array($this->feedid));
	    $this->_dbh->commit();
	    return;
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function refresh() {
	$ch = curl_init($this->url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
	curl_setopt($ch, CURLOPT_PORT, 80);
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_TIMEOUT, 10);
	curl_setopt($ch, CURLOPT_USERAGENT, 'AmalgamateBot 1.0 written by http://amalgamate.take88.com');
	@curl_setopt($ch, CURLOPT_FOLLOWLOCATION,1);
	$xml = curl_exec($ch);
	
	if (!$xml) {
	    return;
	}
	curl_close($ch);
	$unserializer =& new XML_Unserializer();
	$result = $unserializer->unserialize($xml, FALSE, array('returnResult'=>TRUE));

	if (PEAR::isError($result)) {
	    throw $result;
	}
	$data = $unserializer->getUnserializedData();
	$numStories = count($data['channel']['item']);
	$stories = $data['channel']['item'];
	for ($i = 0; $i < $numStories; $i++) {
	    $stories[$i]['feedid'] = $this->feedid;
	    $stories[$i]['content'] = $stories[$i]['description'];
	    try {
		$this->saveStory($stories[$i]);
	    } catch (PDOException $e) {
		throw $e;
	    }
	}
    }

    public function saveStory($story) {
	try {
	    $fstory =& new AmalgamateFeedStory($this->_dbh, FALSE, $story['guid']);
	    $fstory->load($story);
	    $fstory->save();
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function getStories($limit) {
	try {
	    $this->_dbh->beginTransaction();
	    $sth = $this->_dbh->prepare("SELECT * FROM get_feed_stories('stories', ?, ?)");
	    $sth->execute(array($this->feedid, $limit));
	    $sth->closeCursor();
	    $sth = $this->_dbh->prepare('FETCH ALL IN "stories"');
	    $sth->execute();
	    while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
		$s =& new AmalgamateFeedStory($this->_dbh);
		$s->load($row);
		$ret[] = $s;
	    }
	    $sth->closeCursor();
	    $this->_dbh->commit();
	    return $ret;
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    private function load() {
	if (!$this->feedid) return;
	try {
	    $sth = $this->_dbh->prepare("SELECT * FROM get_feed(?)");
	    $sth->execute(array($this->feedid));
	    $row = $sth->fetch(PDO::FETCH_ASSOC);
	    $sth->closeCursor();
	    $this->name = $row['name'];
	    $this->url = $row['url'];
	    $this->createddate = $row['createddate'];
	    $this->lastmodifieddate = $row['lastmodifieddate'];
	    $this->source = $row['source'];
	    $this->active = $row['active'];
	    $this->category = $row['category'];
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function exists() {
	try {
	    $sth = $this->_dbh->prepare("SELECT * FROM feed_exists(?)");
	    $sth->execute(array($this->url));
	    list($feedid) = $sth->fetch(PDO::FETCH_NUM);
	    $sth->closeCursor();
	    return $feedid;
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function save() {
	try {
	    $existingid = $this->exists();
	    if ($existingid) {
		$this->feedid = $existingid;
	    }

	    $this->_dbh->beginTransaction();

	    if ($this->feedid) {
		$sth = $this->_dbh->prepare("SELECT update_feed(?, ?, ?, ?, 't', ?)");
		$sth->execute(array($this->url, $this->name, $this->source, $this->category, $this->feedid));
	    } else {
		$sth = $this->_dbh->prepare("SELECT new_feed(?, ?, ?, ?, 't')");
		$sth->execute(array($this->url, $this->name, $this->source, $this->category));
		list($this->feedid) = $sth->fetch(PDO::FETCH_NUM);
	    }
	    $sth->closeCursor();
	    
	    $this->_dbh->commit();
	    return;
	} catch (PDOException $e) {
	    throw $e;
	}
    }
}

class AmalgamateFeedStory {
    private $_sql;
    private $_dbh;
    public $feedstoryid;
    public $feedid;
    public $title;
    public $content;
    public $guid;
    public $pubdate;
    public $createddate;
    public $lastmodifieddate;
    public $source;
    public $sourcelink;
    
    function __construct(&$dbh, $feedstoryid = FALSE, $guid = FALSE) {
	$this->_dbh = $dbh;
	$this->feedstoryid = $feedstoryid;
	$this->guid = $guid;
	if ($this->feedstoryid) {
	    try {
		$this->loadById();
	    } catch (PDOException $e) {
		throw $e;
	    }
	}
    }

    function loadById() {
	try {
	    $sth = $this->_dbh->prepare("SELECT * FROM get_story(?)");
	    $sth->execute(array($this->feedstoryid));
	    $feed = $sth->fetch(PDO::FETCH_ASSOC);
	    if ($feed['feedstoryid']) {
		$this->load($feed);
	    }
	    $sth->closeCursor();
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    function load($data) {
	if (!$this->feedstoryid) {
	    $this->feedstoryid = $data['feedstoryid'];
	}
	
	$this->feedid = $data['feedid'];
	$this->title = $data['title'];
	$this->content = $data['content'];
	$this->link = $data['link'];
	$this->guid = $data['guid'];
	$this->pubdate = $data['pubdate'];
	$this->createddate = $data['createddate'];
	$this->lastmodifieddate = $data['lastmodifieddate'];
	$this->source = $data['source'];
	$this->sourcelink = $data['sourcelink'];
	$this->name = $data['name'];
	$this->category = $data['category'];
    }

    function save() {
	try {
	    if ($this->feedstoryid) {
		$sth = $this->_dbh->prepare("SELECT update_story(?, ?, ?, ?, ?)");
		$sth->execute(array($this->feedstoryid, strip_tags($this->title, '<a>'), strip_tags($this->content, '<a>'), $this->link));
		$sth->closeCursor();
	    } else {
		$sth = $this->_dbh->prepare("SELECT new_story(?, ?, ?, ?, ?)");
		$sth->execute(array($this->feedid, strip_tags($this->title, '<a>'), strip_tags($this->content, '<a>'), $this->guid, $this->link));
		$sth->closeCursor();
	    }
	} catch (PDOException $e) {
	    throw $e;
	}
    }
    
    public function pubdate() {
	if (!$this->pubdate) {
	    return date('Y-m-d h:i:s');
	}
	
	return date('Y-m-d h:i:s', strtotime($this->pubdate));
    }
}

class AmalgamateException extends Exception {
    public function __construct($message, $code = 0, Exception $previous = null) {
	parent::__construct($message, $code, $previous);
    }
}

class MisterData extends PDO {
    public function __construct($dsn, $username = '', $password = '', $driverOptions = array()) {
	// This traps an exception if MisterData's constructor is called outside of try/catch.
	set_exception_handler(array(__CLASS__, 'exceptionHandler'));
	parent::__construct($dsn, $username, $password, $driverOptions);
	restore_exception_handler();
	// Turn on exceptions for all failures.
	$this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    }

    public static function exceptionHandler($exception) {
	die('Uncaught exception: ' . $exception->getMessage() . "<br />\nWARNING: Please create " . __CLASS__ . " object within a TRY/CATCH.");
    }
}
?>
