-- Load these stored procedures after you load the schema

CREATE LANGUAGE plpgsql;

BEGIN;
-- PL/PGSQL
CREATE OR REPLACE FUNCTION save_source (text, text) RETURNS void AS $$
DECLARE
	src ALIAS FOR $1;
	srclink ALIAS FOR $2;
BEGIN
	INSERT INTO amalgamate_source (source, sourcelink) VALUES(src, srclink);
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_categories () RETURNS SETOF amalgamate_category AS $$
DECLARE
	ret record;
BEGIN
	FOR ret IN SELECT * FROM amalgamate_category ORDER BY category
	LOOP
		RETURN NEXT ret;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_sources() RETURNS SETOF amalgamate_source AS $$
DECLARE
	ret record;
BEGIN
	FOR ret IN SELECT * FROM amalgamate_source ORDER BY source
	LOOP
		RETURN NEXT ret;
	END LOOP;
	RETURN;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_feeds(refcursor) RETURNS refcursor AS $$
BEGIN
	OPEN $1 FOR SELECT a.*, b.source, c.category FROM amalgamate_feed a JOIN amalgamate_source b USING(source) JOIN amalgamate_category c USING(category) WHERE a.active ORDER BY name ASC;
	RETURN $1;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_stories(refcursor, l integer) RETURNS refcursor AS $$
BEGIN
	OPEN $1 FOR SELECT s.source, s.sourcelink, f.feedid, f.url, f.lastupdate, f.createddate, f.active, f.name, f.category, st.title, st.content, st.guid, st.link, st.createddate as pubdate FROM amalgamate_feed_story st JOIN amalgamate_feed f USING(feedid) JOIN amalgamate_source s USING(source) ORDER BY st.feedstoryid DESC LIMIT l;
	RETURN $1;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION delete_feed(id integer) RETURNS void AS $$
BEGIN
	DELETE FROM amalgamate_feed_story WHERE feedid = id;
	DELETE FROM amalgamate_feed WHERE feedid = id;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_feed_stories(refcursor, id integer, l integer) RETURNS refcursor AS $$
BEGIN
	OPEN $1 FOR SELECT s.source, s.sourcelink, f.feedid, f.url, f.lastupdate, f.createddate, f.active, f.name, f.category, st.title, st.content, st.guid, st.link, st.createddate as pubdate FROM amalgamate_feed_story st JOIN amalgamate_feed f USING(feedid) JOIN amalgamate_source s USING(source) WHERE f.feedid = id ORDER BY st.feedstoryid DESC LIMIT l;
	RETURN $1;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION get_feed(id integer) RETURNS amalgamate_feed AS $$
DECLARE
	ret record;
BEGIN
	SELECT * INTO ret FROM amalgamate_feed WHERE feedid = id;
	RETURN ret;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION feed_exists(u text) RETURNS integer AS $$
DECLARE
	ret integer;
BEGIN
	SELECT feedid INTO ret FROM amalgamate_feed WHERE url = u;
	RETURN ret;
END
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION new_feed(u text, n text, src text, c text, a boolean) RETURNS integer AS $$
DECLARE
	ret integer;
BEGIN
	INSERT INTO amalgamate_feed (url, name, source, category, active) VALUES(u, n, src, c, a);
	SELECT CURRVAL('amalgamate_feed_feedid_seq') INTO ret;
	RETURN ret;
END
$$ LANGUAGE plpgsql; 

CREATE OR REPLACE FUNCTION update_feed(u text, n text, src text, c text, a boolean, id integer) RETURNS void AS $$
BEGIN
	UPDATE amalgamate_feed SET url = u, name = n, source = src, category = c, active = a WHERE feedid = id;
	RETURN;
END
$$ LANGUAGE plpgsql; 

CREATE OR REPLACE FUNCTION get_story(id integer) RETURNS SETOF amalgamate_feed_story AS $$
DECLARE
	ret record;
BEGIN
	FOR ret IN SELECT * FROM amalgamate_feed_story WHERE feedstoryid = id
	LOOP
		RETURN NEXT ret;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION new_story(fid integer, t text, c text, g text, l text) RETURNS integer AS $$
DECLARE
	ret integer;
BEGIN
	INSERT INTO amalgamate_feed_story (feedid, title, content, guid, link, createddate) VALUES(fid, c, g, l, g, 'now');
	SELECT CURRVAL('amalgamate_feed_story_feedstoryid_seq') INTO ret;
	RETURN ret;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION update_story(id integer, t text, c text, g text, l text) RETURNS void AS $$

BEGIN
	UPDATE amalgamate_feed_story SET title = t, caption = c, guid = g, link = l WHERE feedstoryid = id;
	RETURN;
END;

$$ LANGUAGE plpgsql;
COMMIT;