-- Before you run this schema
-- 1. createdb amalgamate
-- 2. CREATE LANGUAGE plpgsql;
BEGIN;
DROP FUNCTION save_source(text, text);
DROP FUNCTION get_categories();
DROP FUNCTION get_stories(refcursor, l integer);
DROP FUNCTION get_feeds(refcursor);
DROP FUNCTION delete_story(id integer);
DROP FUNCTION get_feed_stories(refcursor, id integer, l integer);
DROP FUNCTION get_feed(id integer);
DROP TABLE IF EXISTS amalgamate_feed_story CASCADE;
DROP TABLE IF EXISTS amalgamate_feed_tag CASCADE;
DROP TABLE IF EXISTS amalgamate_feed CASCADE;
DROP TABLE IF EXISTS amalgamate_category CASCADE;
DROP TABLE IF EXISTS amalgamate_source CASCADE;

CREATE TABLE amalgamate_source (
    source character varying(100) not null unique,
    primary key(source),
    sourcelink text,
    createddate timestamp without time zone DEFAULT now(),
    lastmodifieddate timestamp without time zone DEFAULT now()
);
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('New York Times', 'http://nytimes');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('Christian Science Monitor', 'http://csmonitor.com');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('Nation', 'http://thenation.com');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('National Review', 'http://www.nationalreview.com');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('Washington Post', 'http://www.washingtonpost.com');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('New Yorker', 'http://www.newyorker.com');
INSERT INTO amalgamate_source (source, sourcelink) VALUES ('Google News', 'http://news.google.com');

CREATE TABLE amalgamate_category (
    category character varying(100) not null unique,
    primary key(category),
    weight integer
);
INSERT INTO amalgamate_category (category) VALUES ('Newspaper');
INSERT INTO amalgamate_category (category) VALUES ('Magazine');
INSERT INTO amalgamate_category (category) VALUES ('Web site');

CREATE TABLE amalgamate_feed (
    feedid serial,
    primary key(feedid),
    url character varying(255),
    lastupdate timestamp without time zone,
    createddate timestamp without time zone DEFAULT now(),
    active boolean,
    name character varying(255),
    source character varying(100) references amalgamate_source on update cascade on delete cascade,
    category character varying(100) references amalgamate_category on update cascade on delete cascade
);

CREATE TABLE amalgamate_feed_story (
    feedstoryid serial,
    primary key(feedstoryid),
    feedid integer references amalgamate_feed on delete cascade,
    title text,
    content text,
    guid text,
    link text,
    createddate timestamp without time zone DEFAULT now()
);

GRANT ALL ON amalgamate_source TO amalgamate;
GRANT ALL ON amalgamate_category TO amalgamate;
GRANT ALL ON amalgamate_feed TO amalgamate;
GRANT ALL ON amalgamate_feed_feedid_seq TO amalgamate;
GRANT ALL ON amalgamate_feed_story TO amalgamate;
GRANT ALL ON amalgamate_feed_story_feedstoryid_seq TO amalgamate;

INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('New York Times', 'http://feeds.nytimes.com/nyt/rss/HomePage', 'Front page', 'Newspaper', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('Nation', 'http://feeds.feedburner.com/ArticlesFromTheNation?format=xml', 'Front page', 'Magazine', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('Christian Science Monitor','http://rss.csmonitor.com/feeds/top', 'Front page', 'Newspaper', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('Washington Post', 'http://feeds.washingtonpost.com/wp-dyn/rss/print/index_xml', 'Front page', 'Newspaper', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('National Review','http://www.nationalreview.com/index.xml', 'Front page', 'Magazine', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('New Yorker', 'http://www.newyorker.com/services/rss/feeds/everything.xml', 'Front page', 'Magazine', 't');
INSERT INTO amalgamate_feed (source, url, name, category, active) VALUES ('Google News', 'http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=open-source&cf=all&output=rss', 'Open source', 'Web site', 't');

COMMIT;
